/*
 *
 * Print or set floppy disk types in the CMOS RAM. Relies on /dev/nvram
 * driver being present in the kernel (or module loaded). To set either
 * drive type you must supply type for both drive types.
 *
 * Compile: gcc -s -Wall cmosfdtype.c -o cmosfdtype
 * Usage: cmosfdtype [drive0 drive1]
 *
 * Tech info:
 *   0x14 bit 7 & 6 are number of floppy drives -1. 0x10 contains the
 *   drive type high nibble is fd0, low nibble is fd1.
 *
 *						Paul Gortmaker, 01/2001.
 *
 * This code is released under the GPL, blah blah blah...
 *
*/

#include <sys/types.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <sys/ioctl.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <errno.h>
#include <fcntl.h>
#include <linux/nvram.h>


/* /dev/nvram excludes 14 registers of RTC */
#define RTC_OFFSET 0x0e

const char *drive_type[] = {
			"Not Installed",
			"360 KB 5 1/4\"",
			"1.2 MB 5 1/4\"",
			"720 KB 3 1/2\"",
			"1.44 MB 3 1/2\"",
			"2.88 MB AMI",
			"2.88 MB"
};
#define N_TYPES 7

void usage(void) {

	int i;

	fprintf(stderr, "Usage: cmosfdtype [type1 type2]\n");
	fprintf(stderr, "\tset floppy disk types in CMOS\n");
	fprintf(stderr, "\ttype1 and type2 are one of:\n");
	for (i=0;i<N_TYPES;i++)
		fprintf(stderr, "\t\t%d:\t%s\n", i, drive_type[i]);

	exit(-EINVAL);
}

int main (int argc, char **argv) {

	int fd, i, fd0, fd1, n_fd;
	unsigned char data, reg_14, ten_high, ten_low;

	fd = open("/dev/nvram", O_RDWR);
	if (fd == -1) {
		perror("open - /dev/nvram");
		exit(errno);
	}

	lseek(fd, 0x0e - RTC_OFFSET, SEEK_SET);
	read(fd, &data, 1);
	if (data != 0) {
		printf("CMOS status byte is non-zero - exiting!\n");
		exit(-1);
	}

	lseek(fd, 0x14 - RTC_OFFSET, SEEK_SET);
	read(fd, &reg_14, 1);
	n_fd = (reg_14>>6)+1;

	printf("Number of floppy drives installed: %d\n", n_fd);

	lseek(fd, 0x10 - RTC_OFFSET, SEEK_SET);
	read(fd, &data, 1);

	ten_high = (data&0xf0)>>4;
	ten_low = data&0x0f;

	if (n_fd >= 1)
		printf("Current Floppy 0: Type %d (%s)\n", ten_high, drive_type[ten_high]);

	if (n_fd > 1)
		printf("Current Floppy 1: Type %d (%s)\n", ten_low, drive_type[ten_low]);

	if (argc != 3) exit(0);

	fd0 = atoi(argv[1]);
	fd1 = atoi(argv[2]);

	if (fd0 >= N_TYPES) usage();
	if (fd1 >= N_TYPES) usage();

	printf("Setting fd0 to type %d (%s), fd1 to type %d (%s)\n",
		 fd0, drive_type[fd0], fd1, drive_type[fd1]);

	data=(fd0<<4) + fd1;

	lseek(fd, 0x10 - RTC_OFFSET, SEEK_SET);
	write(fd, &data, 1);

	n_fd = (fd0 != 0) + (fd1 != 0) - 1;
	data = (reg_14 & ~0xc0) | (n_fd << 6);

	lseek(fd, 0x14 - RTC_OFFSET, SEEK_SET);
	write(fd, &data, 1);

	i = ioctl(fd, NVRAM_SETCKS);
	if (i == -1) {
		perror("ioctl NVRAM_SETCKS - /dev/nvram");
		exit(errno);
	}

	return 0;
}

